//Receives a file from the FileServer.

#include "Dasyne.hpp"
#include <iostream>

using namespace std;
using namespace libdasyne;

//The port number to communicate over
const uint16_t PORT_NUMBER = 7890;

//The number of bytes to allocate in our receive buffer
const unsigned int BUFFER_SIZE = 65535;

//False if we encountered an error and should stop the program
static bool allOK = true;

//Track the time it takes to transfer the file
static DTime transferTime;

//Shows fileclient's command-line usage
void showUsage(void);

class FileClient : public DClient
{
    public:
        //Constructor, we pass the DEngine to the parent DClient class
        FileClient(DEngine *newNetEngine) : DClient(newNetEngine, BUFFER_SIZE)
        {
            haveLength = false;
            fileSize = 0;
        }//constructor

        //Connect to the host
        void connectToHost(string hostname)
        {
            startConnect(hostname, PORT_NUMBER);
        }//connectToHost

        //Event handler, called when the connect operation finishes
        void connectHandler(string message,
                            libdasyne::NetworkCode status)
        {
            if (status == libdasyne::DA_SUCCESS)
                cout << "Connected to " << getRemoteAddress() << endl;
            else
            {
                cout << message << endl;
                allOK = false;
            }//else status
        }//connectHandler

        //Event handler, called when we receive data
        void receiveHandler(DBuffer &receivedBytes)
        {
            fileData.append(receivedBytes);

            //Read the file size
            if (!haveLength)
            {
                if (fileData.size() >= 8)
                {
                    fileData.extractVariable(0, fileSize);
                    fileData.removeFirst(8);
                    haveLength = true;
                    cout << "Expecting a file " << fileSize << " bytes long" << endl;
                }//if size()
            }//if !haveLength

            //Save the file
            //cout << "Received: " << fileData.size() << " / " << fileSize << endl;
            if (fileData.size() == fileSize)
            {
                saveToFile();
                allOK = false;
            }//else if size()
            else if (fileData.size() > fileSize)
            {
                cout << "Received " << fileData.size()
                     << " bytes, was only expecting " << fileSize << endl;
                allOK = false;
            }//else if size()
        }//receiveHandler

        //Event handler, called when a network error occurs
        void errorHandler(string message)
        {
            cout << "Connection lost." << endl;
            allOK = false;
        }//errorHandler

    private:
        //Contains the file we've received
        DBuffer fileData;

        //The length of the file in bytes
        uint64_t fileSize;

        //True if we've received the file length, false otherwise
        bool haveLength;

        //Saves fileData to client.dwnld
        void saveToFile(void)
        {
            DBufferIter dIter;//iterates through fileData
        ofstream outfile;//the file to write to

            cout << "Time to receive file " << transferTime.timeElapsed() << " milliseconds" << endl;

            cout << "Writing to disk as client.dwnld..." << endl;
            outfile.open("client.dwnld", ios::binary);
            if (!outfile.is_open())
                cout << "Couldn't create file!" << endl;
            else
            {
                dIter.setBuffer(&fileData);

                while (!dIter.finished())
                {
                    outfile << dIter.value();
                    dIter.next();
                }//while !finished()

            outfile.close();
            }//else !is_open()
        }//saveToFile
};//FileClient

int main(int argc,
         char *argv[])
{
    DEngine netEngine;//handles network events
    FileClient *client = new FileClient(&netEngine);//receives the file
    int eventsRun = 0;//the number of events dispatched by a call to update()

    transferTime.updateTime();

    //### Connect to the server ###
    if (argc == 2)
        client->connectToHost(argv[1]);
    else
    {
        cout << "Usage: fileclient <hostname>" << endl;
        allOK = false;
    }//else argc

    while (allOK)
    {
        //### Dispatch any queued events ###
        eventsRun = netEngine.update(-1);
        if (eventsRun < 0)
        {
            cout << netEngine.getErrMsg() << endl;
            allOK = false;
        }//if eventsRun

        //### Sleep if we have nothing to do ###
        else if (eventsRun == 0)
            DEngine::sleep(1);
    }//while allOK

    //### Delete the classes that use netEngine before deleting netEngine ###
    delete client;

    return 0;
}//main

void showUsage(void)
{
    cout << "Usage: tclient <host_name> <buffer_KB>" << endl;
    cout << endl;
}//showUsage
