/* An example of the DDispatcher class. We create three messages, each with a different
   length and content, and use the DDispatcher to send them to the appropriate event handler
   in the client. The client's handlers read the contents and verity that they only receive
   their type of message.
*/

#include "Dasyne.hpp"
#include <iostream>

using namespace std;
using namespace libdasyne;
class Client;

const uint16_t PORT_NUMBER = 5137;//The port number to communicate over
static bool allOK = true; //false if we should stop the server
static DBuffer message1;//messages to repeatedly send to connected clients
static DBuffer message219;
static DBuffer message220;
static vector<Client*> theClients;//a list of clients who've connected

class Client : public DDispatcher
{
    public:
        //constructor, passes the network engine and an existing connection to DMessageClient
        Client(DEngine *newNetEngine,
               boost::asio::ip::tcp::socket *newSocket) : DDispatcher(newNetEngine, newSocket)
        {
            setDelay(false);
        }//constructor

    private:
        //Event handler, called when all data is sent
        void sendHandler(void)
        {
            sendMessage(1, message1);
            sendMessage(2, 19, message219);
            sendMessage(2, 20, message220);
        }//sendHandler

        //Event handler called when the connection is lost
        void errorHandler(string message)
        {
            cout << "Client disconnected: " << message << endl;
        }//errorHandler
};//Client

class MessageServer : public DServer
{
    public:
        //Constructor, takes a pointer to a DEngine and passes it to the
        //parent DServer object
        MessageServer(DEngine *newNetEngine) : DServer(newNetEngine)
        {
        }//constructor

    private:
        //Event handler, called when a client connects
        void connectHandler(boost::asio::ip::tcp::socket *newSocket)
        {
            Client *newClient = new Client(netEngine, newSocket);

            cout << "A client connected from " << newClient->getRemoteAddress() << endl;
            theClients.push_back(newClient);
            newClient->sendMessage(1, message1);
        }//connectHandler

        //Event handler, called when there's an error accepting connections
        void errorHandler(string errMsg)
        {
            cout << errMsg << endl;
            allOK = false;
        }//errorHandler
};//MessageServer

int main(int argc,
         char *argv[])
{
    DEngine netEngine;//calls event handlers for networking classes
    MessageServer *myServer = new MessageServer(&netEngine);//listens for client connections
    int eventsRun = 0;//the number of events dispatched by a call to update()

    //### Create the different messages ###
    for (unsigned int i = 0; i < 256; i++)
        message1.appendByte(i);

    for (unsigned int i = 0; i < 50; i++)
        message219.appendByte(i * 2);

    for (unsigned int i = 0; i < 25; i++)
        message220.appendByte(i * 3);

    //### Bind the listen socket ###
    if (!myServer->bind(libdasyne::DA_IPV4, PORT_NUMBER))
    {
        allOK = false;
        cout << "Couldn't bind: " << myServer->getErrMsg() << endl;
    }//if !bind()

    //### Start listening for connections ###
    if (allOK)
    {
        if (myServer->startListening())
            cout << "Listening for TCP connections, hit Ctrl-C to exit." << endl;
        else
        {
            allOK = false;
            cout << "Couldn't listen: " << myServer->getErrMsg() << endl;
        }//else startListening()
    }//if allOK

    //### Periodically call event handlers ###
    while (allOK)
    {
        eventsRun = netEngine.update(-1);
        if (eventsRun < 0)
        {
            cout << netEngine.getErrMsg();
            allOK = false;
        }//if eventsRun

        //Sleep if we have nothing to do
        else if (eventsRun == 0)
            DEngine::sleep(1);
    }//while allOK

    //### Delete the objects that use DEngine before it's deleted ###
    for (unsigned int i = 0; i < theClients.size(); i++)
        delete theClients.at(i);
    theClients.clear();

    delete myServer;

    return 0;
}//main
