/*
 * Copyright (C) 2011 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.hpp
*/

#include "DBufferIter.hpp"
#include "DBytes.hpp"

using namespace std;
using namespace libdasyne;

#ifdef DA_DEBUG_MODE

DBufferIter::DBufferIter(void) : bytePointer(1, false)
{
    initVars();
}//constructor

DBufferIter::DBufferIter(const DBufferIter &source) : bytePointer(1, false)
{
    copyVars(source);
}//copy constructor

#else

DBufferIter::DBufferIter(void)
{
    initVars();
}//constructor

DBufferIter::DBufferIter(const DBufferIter &source)
{
    copyVars(source);
}//copy constructor

#endif

DBufferIter::~DBufferIter(void)
{
}//destructor

void DBufferIter::initVars(void)
{
    #ifndef DA_DEBUG_MODE
        bytePointer = NULL;
    #endif

    theBuffer = NULL;
    dataIndex = 0;
    arrayIndex = 0;
    maxArrayIndex = 0;
    passedEnd = false;
}//initVars

void DBufferIter::copyVars(const DBufferIter &source)
{
    bytePointer = source.bytePointer;
    theBuffer = source.theBuffer;
    dataIndex = source.dataIndex;
    arrayIndex = source.dataIndex;
    maxArrayIndex = source.maxArrayIndex;
    passedEnd = source.passedEnd;
}//copyVars

DBufferIter& DBufferIter::operator= (const DBufferIter &source)
{
    if (this != &source)
        copyVars(source);

    return *this;
}//=

void DBufferIter::setBuffer(const DBytes *newBuffer)
{
    theBuffer = newBuffer;
    setIndex(0);
}//setBuffer

void DBufferIter::setIndex(long int newIndex)
{
    passedEnd = false;

    //### Verify that we have a buffer ###
    if (theBuffer == NULL)
        DEngine::fatalError("DBufferIter::setIndex(): buffer is NULL");

    //### Verify that the starting index is within bounds ###
    if (newIndex < 0)
        DEngine::fatalError("DBufferIter::setIndex(): index is negative");
    else if (newIndex >= static_cast<long int>(theBuffer->size()))
    {
        if (newIndex == 0)
            passedEnd = true;
        else
            DEngine::fatalError("DBufferIter::setIndex(): out of bounds");
    }//else if newIndex

    //### Find the desired byte array ###
    theBuffer->calcIndexes(newIndex, dataIndex, arrayIndex);
    bytePointer = theBuffer->byteData.at(dataIndex);
    maxArrayIndex = DBytes::ARRAY_SIZE - 1;

    if (dataIndex == theBuffer->byteData.size() - 1)
    {
        if (theBuffer->byteData.size() == 1)
        {
            maxArrayIndex = arrayIndex + (theBuffer->size() - 1);
            if (maxArrayIndex >= static_cast<uint16_t>(DBytes::ARRAY_SIZE))
                DEngine::fatalError("DBufferIter::setIndex(): maxArrayIndex > ARRAY_SIZE");
        }//if size()
        else
            maxArrayIndex = theBuffer->endFilledBytes - 1;
    }//if dataIndex
}//setIndex

uint8_t& DBufferIter::value(void) const
{
    return bytePointer[arrayIndex];
}//getByte

void DBufferIter::next(void)
{
    arrayIndex++;
    if (arrayIndex > maxArrayIndex)
    {
        dataIndex++;
        arrayIndex = 0;
        maxArrayIndex = DBytes::ARRAY_SIZE - 1;

        if (dataIndex <= theBuffer->byteData.size() - 1)
            bytePointer = theBuffer->byteData.at(dataIndex);

        if (dataIndex == theBuffer->byteData.size() - 1)
            maxArrayIndex = theBuffer->endFilledBytes - 1;

        else if (dataIndex >= theBuffer->byteData.size())
            passedEnd = true;
    }//if arrayIndex
}//next

bool DBufferIter::finished(void)
{
    return passedEnd;
}//finished
