/*
 * Copyright (C) 2005-2011 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.hpp
*/

#include "DMutex.hpp"

using namespace std;
using namespace libdasyne;

#ifdef DA_WINDOWS_VERSION

DMutex::DMutex(void)
{
    theMutex = CreateMutex(NULL, FALSE, NULL);
    if (theMutex == NULL)
    {
        errMsg = "DMutex::DMutex() CreateMutex failed";
        //errMsg.append(GetLastError());

        throw 1;
    }//if theMutex
}//constructor

DMutex::~DMutex(void)
{
    if (!CloseHandle(theMutex))
    {
        errMsg = "DMutex::~DMutex() CloseHandle failed";
        //errMsg.append(GetLastError());

        throw 1;
    }//if !CloseHandle
}//desctructor

void DMutex::lock(void)
{
    if (WaitForSingleObject(theMutex, INFINITE) != WAIT_OBJECT_0)
    {
        errMsg = "DMutex::lock() failed!";
        throw 1;
    }//if WaitForSingleObject()
}//lock

void DMutex::unlock(void)
{
    if (!ReleaseMutex(theMutex))
    {
        errMsg = "DMutex::unlock() failed!";
        throw 1;
    }//if !ReleaseMutex()
}//unlock

#else

DMutex::DMutex(void)
{
    if (pthread_mutex_init(&theMutex, NULL) != 0)
    {
        errMsg = "DMutex::DMutex() pthread_mutex_init failed";
        throw 1;
    }//if pthread_mutex_init()
}//constructor

DMutex::~DMutex(void)
{
    if (pthread_mutex_destroy(&theMutex) != 0)
    {
        errMsg = "DMutex::~DMutex() pthread_mutex_destroy failed";
        throw 1;
    }//if pthread_mutex_destroy()
}//desctructor

void DMutex::lock(void)
{
    if (pthread_mutex_lock(&theMutex) != 0)
    {
        errMsg = "DMutex::lock() failed!";
        throw 1;
    }//if pthread_mutex_lock()
}//enterLock

void DMutex::unlock(void)
{
    if (pthread_mutex_unlock(&theMutex) != 0)
    {
        errMsg = "DMutex::unlock() failed!";
        throw 1;
    }//if pthread_mutex_unlock()
}//unlock

#endif

string DMutex::getErrMsg(void)
{
    return errMsg;
}//getErrMsg
