/*
 * Copyright (C) 2011 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.h
*/

#include "DSafeArray.hpp"

using namespace std;
using namespace libdasyne;

long int DSafeArray::numInstances = 0;

DSafeArray::DSafeArray(long int newSize,
                       bool newIsTemporary)
{
    numInstances++;

    isTemporary = newIsTemporary;
    isShallowCopy = false;

    if (isTemporary)
    {
        theBytes = NULL;
        temporarySize = newSize;
    }//if isTemporary
    else
    {
        temporarySize = 1;
        theBytes = new vector<uint8_t>;
        theBytes->resize(newSize, 100);
    }//else isTemporary
}//constructor

DSafeArray::DSafeArray(const DSafeArray &source)
{
    numInstances++;

    isTemporary = source.isTemporary;
    temporarySize = source.temporarySize;

    if (isTemporary)
    {
        theBytes = NULL;
        isShallowCopy = false;
    }//if isTemporary
    else
    {
        theBytes = source.theBytes;
        isShallowCopy = true;
    }//else isTemporary
}//copy constructor

DSafeArray::~DSafeArray(void)
{
    numInstances--;

    if (!isShallowCopy)
        delete theBytes;
}//destructor

void DSafeArray::makePermanent(void)
{
    if (!isTemporary)
        throw 1;

    if (theBytes != NULL)
        throw 1;

    theBytes = new vector<uint8_t>;
    theBytes->resize(temporarySize, 100);
    isTemporary = false;
}//makePermanent

uint8_t& DSafeArray::operator [] (long int index)
{
    return theBytes->at(index);
}//[]

DSafeArray& DSafeArray::operator = (const DSafeArray &source)
{
    if (!isShallowCopy)
        delete theBytes;
    theBytes = source.theBytes;

    isShallowCopy = true;

    return *this;
}//=

long int DSafeArray::getNumInstances(void)
{
    return numInstances;
}//getNumInstances
