/*
 * Copyright (C) 2011 David C. McCallum
 * For conditions of distribution and use, see the copyright notice in Dasyne.hpp
*/

#include "DTimer.hpp"
#include "DTime.hpp"
#include "DTimerEvent.hpp"

using namespace std;
using namespace libdasyne;

DTimer::DTimer(DEngine *newEngine)
{
    engine = newEngine;
    eventHandler = NULL;

    myTime = new DTime;
    myTime->useEngine(engine);

    interval = 0;
    running = false;
    haveHandler = false;
}//constructor

DTimer::DTimer(const DTimer &source)
{
    myTime = NULL;
    eventHandler = NULL;
    copyVariables(source);
}//copy constructor

DTimer::~DTimer(void)
{
    delete myTime;
    myTime = NULL;

    delete eventHandler;
    eventHandler = NULL;
}//destructor

DTimer& DTimer::operator= (const DTimer &source)
{
    if (this != &source)
        copyVariables(source);

    return *this;
}//=

void DTimer::copyVariables(const DTimer &source)
{
    if (myTime != NULL)
        delete myTime;
    myTime = new DTime(*source.myTime);

    if (eventHandler != NULL)
        delete eventHandler;
    if (source.eventHandler != NULL)
        eventHandler = new DTimerEvent(source.engine, this);

    engine = source.engine;
    interval = source.interval;
    running = source.running;
    haveHandler = source.haveHandler;
    if (haveHandler)
        handler = source.handler;
}//copyVariables

void DTimer::setInterval(unsigned int newInterval)
{
    interval = newInterval;
    if (interval == 0)
        stop();
}//setInterval

void DTimer::start(void)
{
    if (interval != 0)
    {
        if (eventHandler == NULL)
            eventHandler = new DTimerEvent(engine, this);

        myTime->updateTime();
        running = true;
    }//if interval
}//start

void DTimer::stop(void)
{
    running = false;

    delete eventHandler;
    eventHandler = NULL;
}//stop

unsigned int DTimer::update(void)
{
    unsigned int result = 0;//the number of events dispatched

    if (running)
    {
        while ((myTime->timeElapsed() >= interval) && (running))
        {
            myTime->addTime(interval);

            if (haveHandler)
                handler();
            else
                elapsed();

            result++;
        }//while timeElapsed() && running
    }//if running

    return result;
}//update

unsigned int DTimer::getInterval(void)
{
    return interval;
}//getInterval

bool DTimer::isRunning(void)
{
    return running;
}//isRunning

void DTimer::setHandler(boost::function<void (void)> newHandler)
{
    haveHandler = true;
    handler = newHandler;
}//setHandler

void DTimer::elapsed(void)
{
}//elapsed
